use AllNasdaqTickerPricesfrom2014into2017
go


-- create fresh copy of rsi table
-- with symbol and symbol_number columns
begin try
drop table rsi
end try
begin catch
print 'rsi not available to drop'
end catch


CREATE TABLE [dbo].[rsi](
	[symbol] [varchar](10) NULL,
	[date] [date] NULL,
	[close] [money] NULL,
	[relative strength (rs)] [real] NULL,
	[rsi] [real] NULL
) ON [PRIMARY]



----------------------------------------------------------------------------------------------------

-- declare local variables for Relative Strength
-- and Relative Strength Index (rsi) computation
declare @date_cnt int 
,@row_number int
,@gain_prior money
,@loss_prior money
,@avg_gain_prior money
,@avg_loss_prior money
,@gain_current money
,@loss_current money
,@avg_gain_current money
,@avg_loss_current money
,@rs_prior real
,@rsi_prior real
,@rs_current real
,@rsi_current real


----------------------------------------------------------------------------------------------------


-- create fresh copy of ##symbol table
-- with symbol and symbol_number columns
begin try
drop table ##symbol
end try
begin catch
print '##symbol not available to drop'
end catch

select 
[symbol]
,row_number() over (order by symbol) AS symbol_number
into ##symbol
from
(
select distinct symbol
from  Results_with_extracted_casted_values
) for_distinct_symbols
order by symbol





-- declare local variables to help
-- loop through stock symbols
declare @maxPK int;Select @maxPK = max(symbol_number) From ##symbol
declare @pk int = 1
declare @symbol varchar(5)


-- start while loop for successive @symbol values
while @pk <= @maxPK
begin

-- set @symbol for current pass through loop
set @symbol = (select [symbol] from ##symbol where symbol_number =  @pk)


----------------------------------------------------------------------------------------------------

-- create #for_all_avg_gain_loss
-- populate row 15 for avg_gain and avg_loss
-- configure for all subsequent rows

begin try
drop table #for_all_avg_gain_loss
end try
begin catch
print '#for_all_avg_gain_loss not available to drop'
end catch


-- for current @symbol value
-- create a fresh copy of #for_all_avg_gain_loss
-- assign prime values for row 15
-- and configure table for updating 
select 
row_number() over (order by date) row_number
,symbol
,[date]
,[close]
,round(isnull([close] 
	- lag([close],1) over (order by date),0),5) change
,round(isnull(gain,0),5) gain
,round(isnull(abs(loss),0),5) loss
,
round(
case
	when row_number <= 14 then null
	when row_number = 15 then 
		avg(round(isnull(gain,0),5)) over
		(order by [date] rows between 13 preceding and current row) 
end 
,5) avg_gain
,
round(
case
	when row_number <= 14 then null
	when row_number = 15 then 
		avg(round(isnull(abs(loss),0),5)) over
		(order by [date] rows between 13 preceding and current row) 
end 
,5) avg_loss
,cast(NULL as real) [relative strength (rs)]
,cast(NULL as real) [rsi]
into #for_all_avg_gain_loss
from
(
-- for gains and losses
select 
symbol
,[date]
,row_number() over (order by [date]) row_number
,[close]
,[close]-(lag([close]) over(order by date)) change
,
case
when ([close]-(lag([close]) over(order by date))) > 0
	then [close]-(lag([close]) over(order by date))
end gain
,
case
when ([close]-(lag([close]) over(order by date))) < 0
	then [close]-(lag([close]) over(order by date))
end loss
from Results_with_extracted_casted_values
where symbol = @symbol
) for_15_row_avg_gain_loss


----------------------------------------------------------------------------------------------------

-- initialize local variables for @symbol pass through loop
set @date_cnt = (select count(date) from #for_all_avg_gain_loss where symbol = @symbol)
set @row_number = 15

----------------------------------------------------------------------------------------------------

-- start looping through rows after 14th row until last row
-- process rows iteratively two at a time
while @row_number > 14 and @row_number <= 
	(select count(date) from #for_all_avg_gain_loss)  
begin

-- assign values to local variables for prior and current rows
-- initially 15 denotes prior row and 16 denotes current row
-- pairs of rows are successively (15,16), (16,17), (17,18)...
select
 @gain_prior = isnull((select gain from #for_all_avg_gain_loss 
				where row_number = @row_number),0)
,@loss_prior = abs(isnull((select loss from #for_all_avg_gain_loss 
				where row_number = @row_number),0)) 
,@avg_gain_prior = (select avg_gain from #for_all_avg_gain_loss 
				where row_number = @row_number)
,@avg_loss_prior  = abs((select avg_loss from #for_all_avg_gain_loss 
				where row_number = @row_number))
,@gain_current = isnull((select gain from #for_all_avg_gain_loss 
				where row_number = @row_number + 1),0)
,@loss_current = abs(isnull((select loss from #for_all_avg_gain_loss 
				where row_number = @row_number + 1),0))
,@avg_gain_current = (@avg_gain_prior*13 + @gain_current)/14
,@avg_loss_current = (@avg_loss_prior*13 + @loss_current)/14


-- update prior row
update #for_all_avg_gain_loss
set 
	[relative strength (rs)] = 
	 case when @avg_loss_prior != 0 
	 then @avg_gain_prior/@avg_loss_prior 
	 else 1 
	 end 
	,rsi = 
	 case
	 when @avg_loss_prior != 0 then 
		(100 - (100/(1+(@avg_gain_prior/@avg_loss_prior))))
	 else
	 100
	 end
where #for_all_avg_gain_loss.row_number = @row_number

-- update current row
update #for_all_avg_gain_loss
set 
	 avg_gain = ((@avg_gain_prior*13) + @gain_current)/14
	,avg_loss = ((@avg_loss_prior*13) + @loss_current)/14
	,[relative strength (rs)] = case
		when 
		-- additional error checks for handling missing data
		-- for selected symbols
		((@avg_loss_prior*13) + @loss_current) is not null 
		and (((@avg_gain_prior*13) + @gain_current)/14) is not null 
		and (((@avg_loss_prior*13) + @loss_current)/14) != 0 then 
		(((@avg_gain_prior*13) + @gain_current)/14)
		/(((@avg_loss_prior*13) + @loss_current)/14)
		else null
	 end
	,rsi = 
	 case
	 when @avg_loss_current != 0 then 
		(100 - (100/(1+(@avg_gain_current/@avg_loss_current))))
	 else
	 100
	 end
where #for_all_avg_gain_loss.row_number = @row_number+1

-- increment @row_number by 1 to get to
-- next to rows to process for rsi
set @row_number = @row_number + 1

end

----------------------------------------------------------------------------------------------------

insert into rsi
select 
 symbol
,date
,[close]
,[relative strength (rs)]
,rsi
from #for_all_avg_gain_loss
order by date


-- update @pk value for next set of 
-- @symbol exponential moving averages 
    Select @pk = @pk + 1

end


